@@ -36,8 +36,8 @@
 
 /* Properties */
 enum {
-        PROP_0,
-        PROP_VIEWER
+	PROP_0,
+	PROP_VIEWER
 };
 
 
@@ -53,6 +53,12 @@ struct _GthImageNavigatorPrivate {
 };
 
 
+G_DEFINE_TYPE_WITH_CODE (GthImageNavigator,
+			 gth_image_navigator,
+			 GTK_TYPE_CONTAINER,
+			 G_ADD_PRIVATE (GthImageNavigator))
+
+
 static void
 _gth_image_navigator_set_viewer (GthImageNavigator *self,
 			         GtkWidget         *viewer)
@@ -191,6 +197,7 @@ gth_image_navigator_size_allocate (GtkWidget     *widget,
 	}
 
 	gtk_widget_size_allocate (self->priv->viewer, &viewer_allocation);
+	gtk_widget_queue_draw (widget);
 }
 
 
@@ -295,8 +302,6 @@ gth_image_navigator_class_init (GthImageNavigatorClass *klass)
 	GtkWidgetClass    *widget_class;
 	GtkContainerClass *container_class;
 
-	g_type_class_add_private (klass, sizeof (GthImageNavigatorPrivate));
-
 	object_class = (GObjectClass *) klass;
 	object_class->set_property = gth_image_navigator_set_property;
 	object_class->get_property = gth_image_navigator_get_property;
@@ -474,8 +479,6 @@ navigator_event_area_button_press_event_cb (GtkWidget      *widget,
 	nav_popup->y_root = event->y_root;
 	nav_popup->window_moved = FALSE;
 
-	gtk_window_set_wmclass (GTK_WINDOW (nav_popup->popup_win), "", "gthumb_navigator");
-
 	out_frame = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME (out_frame), GTK_SHADOW_OUT);
 	gtk_container_add (GTK_CONTAINER (nav_popup->popup_win), out_frame);
@@ -505,11 +508,9 @@ gth_image_navigator_init (GthImageNavigator *self)
 {
 	GtkWidget *navigator_icon;
 
-	self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self), GTH_TYPE_IMAGE_NAVIGATOR, GthImageNavigatorPrivate);
-
 	gtk_widget_set_has_window (GTK_WIDGET (self), FALSE);
-	gtk_container_set_reallocate_redraws (GTK_CONTAINER (self), TRUE);
 
+	self->priv = gth_image_navigator_get_instance_private (self);
 	self->priv->automatic_scrollbars = TRUE;
 	self->priv->hscrollbar_visible = FALSE;
 	self->priv->vscrollbar_visible = FALSE;
@@ -543,9 +544,6 @@ gth_image_navigator_init (GthImageNavigator *self)
 }
 
 
-G_DEFINE_TYPE (GthImageNavigator, gth_image_navigator, GTK_TYPE_CONTAINER)
-
-
 GtkWidget *
 gth_image_navigator_new (GthImageViewer *viewer)
 {
